@@ -4,6 +4,7 @@
 """
 
 import abc
+import dataclasses
 import datetime
 import io
 import json
@@ -16,6 +17,7 @@ import socket
 import stat
 import struct
 import subprocess
+import tempfile
 import textwrap
 import threading
 import time
@@ -34,7 +36,7 @@ from retry import retry
 
 from ._adb import AdbConnection, BaseClient
 from ._proto import *
-from ._proto import StrOrPathLike
+from ._proto import StrOrPathLike, AppInfo
 from ._utils import (APKReader, ReadProgress, StopEvent, adb_path,
                      get_free_port, humanize, list2cmdline)
 from ._version import __version__
@@ -394,7 +396,8 @@ class BaseDevice:
         event = StopEvent()
         stream = self.shell(command, stream=True)
         fdst = pathlib.Path(file).open("w", encoding="UTF-8")
-        threading.Thread(target=_copy2file,
+        threading.Thread(name="logcat",
+                         target=_copy2file,
                          args=(stream, fdst, event, _filter_func),
                          daemon=True).start()
         return event
@@ -616,22 +619,29 @@ class AdbDevice(BaseDevice):
     def _prepare(self):
         self._record_client = None
 
-    def screenshot(self) -> Image.Image:
-        """ not thread safe """
-        try:
-            conn = self.shell(["screencap", "-p"], stream=True)
-            raw_png = b''
-            while True:
-                chunk = conn.read(4096)
-                if not chunk:
-                    break
-                raw_png += chunk
-            im = Image.open(io.BytesIO(raw_png))
-            self._width, self._height = im.size
+    def __screencap(self) -> Image.Image:
+        inner_tmp_path = "/sdcard/adbutils-tmp001.png"
+        self.shell(["screencap", "-p", inner_tmp_path])
+
+        with tempfile.TemporaryDirectory() as tmpdir:
+            target_path = os.path.join(tmpdir, "tmp001.png")
+            self.sync.pull(inner_tmp_path, target_path)
+            self.shell(['rm', inner_tmp_path])
+            im = Image.open(target_path)
+            im.load()
             return im.convert("RGB")
-        except UnidentifiedImageError:
-            w, h = self.window_size()
-            return Image.new("RGB", (w, h), (220, 120, 100))
+            
+    def screenshot(self) -> Image.Image:
+        """ not thread safe
+        
+        Note:
+            screencap to file and pull is more stable then shell(stream=True)
+            Ref: https://github.com/openatx/adbutils/pull/78
+        """
+        try:
+            return self.__screencap()
+        except UnidentifiedImageError as e:
+            raise AdbError("screenshot with screencap error", e)
 
     def switch_screen(self, status: bool):
         """
@@ -795,7 +805,8 @@ class AdbDevice(BaseDevice):
                 nolaunch: bool = False,
                 uninstall: bool = False,
                 silent: bool = False,
-                callback: typing.Callable[[str], None] = None):
+                callback: typing.Callable[[str], None] = None,
+                flags: list = ["-r", "-t"]):
         """
         Install APK to device
 
@@ -805,7 +816,8 @@ class AdbDevice(BaseDevice):
             uninstall: uninstall app before install
             silent: disable log message print
             callback: only two event now: <"BEFORE_INSTALL" | "FINALLY">
-        
+            flags (list): default ["-r", "-t"]
+
         Raises:
             AdbInstallError, BrokenPipeError
         """
@@ -864,7 +876,7 @@ class AdbDevice(BaseDevice):
             if callback:
                 callback("BEFORE_INSTALL")
 
-            self.install_remote(dst, clean=True)
+            self.install_remote(dst, clean=True, flags=flags)
             _dprint("Success installed, time used %d seconds" %
                     (time.time() - start))
             if not nolaunch:
@@ -879,7 +891,7 @@ class AdbDevice(BaseDevice):
             ]:
                 _dprint("uninstall %s because %s" % (package_name, e.reason))
                 self.uninstall(package_name)
-                self.install_remote(dst, clean=True)
+                self.install_remote(dst, clean=True, flags=flags)
                 _dprint("Success installed, time used %d seconds" %
                         (time.time() - start))
                 if not nolaunch:
@@ -891,7 +903,7 @@ class AdbDevice(BaseDevice):
                     # ])
             elif e.reason == "INSTALL_FAILED_CANCELLED_BY_USER":
                 _dprint("Catch error %s, reinstall" % e.reason)
-                self.install_remote(dst, clean=True)
+                self.install_remote(dst, clean=True, flags=flags)
                 _dprint("Success installed, time used %d seconds" %
                         (time.time() - start))
             else:
@@ -948,6 +960,10 @@ class AdbDevice(BaseDevice):
             result.append(m.group(1))
         return list(sorted(result))
 
+    @deprecated(deprecated_in="1.1.2",
+                removed_in="2.0.0",
+                current_version=__version__,
+                details="Use app_info instead")
     def package_info(self, package_name: str) -> typing.Union[dict, None]:
         """
         version_code might be empty
@@ -955,37 +971,10 @@ class AdbDevice(BaseDevice):
         Returns:
             None or dict(version_name, version_code, signature)
         """
-        output = self.shell(['dumpsys', 'package', package_name])
-        m = re.compile(r'versionName=(?P<name>[\w.]+)').search(output)
-        version_name = m.group('name') if m else ""
-        m = re.compile(r'versionCode=(?P<code>\d+)').search(output)
-        version_code = m.group('code') if m else ""
-        if version_code == "0":
-            version_code = ""
-        m = re.search(r'PackageSignatures\{.*?\[(.*)\]\}', output)
-        signature = m.group(1) if m else None
-        if not version_name and signature is None:
-            return None
-        m = re.compile(r"pkgFlags=\[\s*(.*)\s*\]").search(output)
-        pkgflags = m.group(1) if m else ""
-        pkgflags = pkgflags.split()
-
-        time_regex = r"[-\d]+\s+[:\d]+"
-        m = re.compile(f"firstInstallTime=({time_regex})").search(output)
-        first_install_time = datetime.datetime.strptime(
-            m.group(1), "%Y-%m-%d %H:%M:%S") if m else None
-
-        m = re.compile(f"lastUpdateTime=({time_regex})").search(output)
-        last_update_time = datetime.datetime.strptime(
-            m.group(1).strip(), "%Y-%m-%d %H:%M:%S") if m else None
-
-        return dict(package_name=package_name,
-                    version_name=version_name,
-                    version_code=version_code,
-                    flags=pkgflags,
-                    first_install_time=first_install_time,
-                    last_update_time=last_update_time,
-                    signature=signature)
+        app_info = self.app_info(package_name)
+        if app_info is None:
+            return app_info
+        return dataclasses.asdict(app_info)
 
     def rotation(self) -> int:
         """
@@ -1061,6 +1050,52 @@ class AdbDevice(BaseDevice):
     def app_clear(self, package_name: str):
         self.shell(["pm", "clear", package_name])
 
+    def app_info(self, package_name: str) -> typing.Optional[AppInfo]:
+        """
+        Get app info
+
+        Returns:
+            None or AppInfo
+        """
+        output = self.shell(['pm', 'path', package_name])
+        if "package:" not in output:
+            return None
+        apk_path = output.split(":", 1)[-1].strip()
+        output = self.shell(['dumpsys', 'package', package_name])
+        m = re.compile(r'versionName=(?P<name>[\w.]+)').search(output)
+        version_name = m.group('name') if m else ""
+        if version_name == "null": # Java dumps "null" for null values
+            version_name = None
+        m = re.compile(r'versionCode=(?P<code>\d+)').search(output)
+        version_code = m.group('code') if m else ""
+        version_code = int(version_code) if version_code.isdigit() else None
+        m = re.search(r'PackageSignatures\{.*?\[(.*)\]\}', output)
+        signature = m.group(1) if m else None
+        if not version_name and signature is None:
+            return None
+        m = re.compile(r"pkgFlags=\[\s*(.*)\s*\]").search(output)
+        pkgflags = m.group(1) if m else ""
+        pkgflags = pkgflags.split()
+
+        time_regex = r"[-\d]+\s+[:\d]+"
+        m = re.compile(f"firstInstallTime=({time_regex})").search(output)
+        first_install_time = datetime.datetime.strptime(
+            m.group(1), "%Y-%m-%d %H:%M:%S") if m else None
+
+        m = re.compile(f"lastUpdateTime=({time_regex})").search(output)
+        last_update_time = datetime.datetime.strptime(
+            m.group(1).strip(), "%Y-%m-%d %H:%M:%S") if m else None
+
+        app_info = AppInfo(package_name=package_name,
+                    version_name=version_name,
+                    version_code=version_code,
+                    flags=pkgflags,
+                    first_install_time=first_install_time,
+                    last_update_time=last_update_time,
+                    signature=signature,
+                    path=apk_path)
+        return app_info
+
     def is_screen_on(self):
         output = self.shell(["dumpsys", "power"])
         return 'mHoldingDisplaySuspendBlocker=true' in output
@@ -1091,21 +1126,6 @@ class AdbDevice(BaseDevice):
             buf += chunk
         return buf.decode("utf-8")
 
-    @deprecated(deprecated_in="0.15.0",
-                removed_in="1.0.0",
-                current_version=__version__,
-                details="Use app_current instead")
-    def current_app(self) -> dict:
-        """
-        Returns:
-            dict(package, activity, pid?)
-
-        Raises:
-            AdbError
-        """
-        info = self.app_current()
-        return asdict(info)
-
     @retry(AdbError, delay=.5, tries=3, jitter=.1)
     def app_current(self) -> RunningAppInfo:
         """
@@ -1213,7 +1233,7 @@ class _ScrcpyScreenRecord(AbstractScreenRecord):
         env['ANDROID_SERIAL'] = self._d.serial
         self._p = subprocess.Popen([self._scrcpy_path, '--no-control', '--no-display', '--record', filename],
                                    stdin=subprocess.DEVNULL,
-                                   stdout=subprocess.DEVNULL, env=env)  # yapf: disable
+                                   stdout=subprocess.DEVNULL, env=env)
         self._finalizer = weakref.finalize(self._p, self._p.kill)
 
     def _stop(self):
